"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.grafserv = exports.NodeGrafserv = void 0;
const node_querystring_1 = require("node:querystring");
const base_js_1 = require("../../core/base.js");
const utils_js_1 = require("../../utils.js");
class NodeGrafserv extends base_js_1.GrafservBase {
    constructor(config) {
        super(config);
    }
    getDigest(dynamicOptions, req, res, isHTTPS) {
        const reqUrl = req.url;
        const qi = reqUrl.indexOf("?");
        const path = qi >= 0 ? reqUrl.substring(0, qi) : reqUrl;
        const search = qi >= 0 ? reqUrl.substring(qi + 1) : null;
        return {
            httpVersionMajor: req.httpVersionMajor,
            httpVersionMinor: req.httpVersionMinor,
            isSecure: isHTTPS,
            method: req.method,
            path,
            headers: (0, utils_js_1.processHeaders)(req.headers),
            getQueryParams() {
                const queryParams = search
                    ? (0, node_querystring_1.parse)(search)
                    : Object.create(null);
                return queryParams;
            },
            getBody() {
                return (0, utils_js_1.getBodyFromRequest)(req, dynamicOptions.maxRequestLength);
            },
            requestContext: {
                node: {
                    req,
                    res,
                },
            },
        };
    }
    createHandler(isHTTPS = false) {
        const dynamicOptions = this.dynamicOptions;
        // FIXME: 'async' here is risky
        return async (req, res, next) => {
            try {
                const request = this.getDigest(dynamicOptions, req, res, isHTTPS);
                const result = await this.processRequest(request);
                if (result === null) {
                    if (typeof next === "function") {
                        return next();
                    }
                    else {
                        const payload = Buffer.from(`Could not process ${req.method} request to ${req.url} ─ please POST requests to ${dynamicOptions.graphqlPath}`, "utf8");
                        res.writeHead(404, {
                            "Content-Type": "text/plain; charset=utf-8",
                            "Content-Length": payload.length,
                        });
                        res.end(payload);
                        return;
                    }
                }
                switch (result.type) {
                    case "error": {
                        if (typeof next === "function") {
                            return next(result.error);
                        }
                        else {
                            const payload = Buffer.from("An error occurred", "utf8");
                            res.writeHead(500, {
                                "Content-Type": "text/plain; charset=utf-8",
                                "Content-Length": payload.length,
                            });
                            res.end(payload);
                            return;
                        }
                    }
                    case "buffer": {
                        const { statusCode, headers, buffer } = result;
                        res.writeHead(statusCode, headers);
                        res.end(buffer);
                        return;
                    }
                    case "json": {
                        const { statusCode, headers, json } = result;
                        const buffer = Buffer.from(JSON.stringify(json), "utf8");
                        headers["Content-Length"] = String(buffer.length);
                        res.writeHead(statusCode, headers);
                        res.end(buffer);
                        return;
                    }
                    case "bufferStream": {
                        const { statusCode, headers, lowLatency, bufferIterator } = result;
                        if (lowLatency) {
                            req.socket.setTimeout(0);
                            req.socket.setNoDelay(true);
                            req.socket.setKeepAlive(true);
                        }
                        res.writeHead(statusCode, headers);
                        // Clean up when connection closes.
                        const cleanup = () => {
                            try {
                                bufferIterator.return?.();
                            }
                            catch {
                                /* nom nom nom */
                            }
                            req.removeListener("close", cleanup);
                            req.removeListener("finish", cleanup);
                            req.removeListener("error", cleanup);
                        };
                        req.on("close", cleanup);
                        req.on("finish", cleanup);
                        req.on("error", cleanup);
                        // https://github.com/expressjs/compression#server-sent-events
                        const flush = lowLatency
                            ? typeof res.flush === "function"
                                ? res.flush.bind(res)
                                : typeof res.flushHeaders === "function"
                                    ? res.flushHeaders.bind(res)
                                    : null
                            : null;
                        try {
                            for await (const buffer of bufferIterator) {
                                res.write(buffer);
                                // FIXME: Technically we should see if `.write()` returned
                                // false, and if so we should pause the stream.
                                if (flush) {
                                    flush();
                                }
                            }
                        }
                        catch (e) {
                            console.error(`Error occurred during stream; swallowing error.`, e);
                        }
                        finally {
                            res.end();
                        }
                        return;
                    }
                    default: {
                        const never = result;
                        console.log("Unhandled:");
                        console.dir(never);
                        const payload = Buffer.from("Server hasn't implemented this yet", "utf8");
                        res.writeHead(503, { "Content-Length": payload.length });
                        res.end(payload);
                        return;
                    }
                }
            }
            catch (e) {
                console.error("Unexpected error occurred:");
                console.error(e);
                if (typeof next === "function") {
                    next(e);
                }
                else {
                    const text = "Unknown error occurred";
                    res.writeHead(500, {
                        "Content-Type": "text/plain",
                        "Content-Length": text.length,
                    });
                    res.end(text);
                }
            }
        };
    }
}
exports.NodeGrafserv = NodeGrafserv;
function grafserv(config) {
    return new NodeGrafserv(config);
}
exports.grafserv = grafserv;
//# sourceMappingURL=index.js.map