"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.httpError = exports.normalizeRequest = exports.memo = exports.getBodyFromFrameworkBody = exports.getBodyFromRequest = exports.processHeaders = exports.handleErrors = void 0;
const grafast_1 = require("grafast");
const graphql_1 = require("graphql");
const interfaces_js_1 = require("./interfaces.js");
function handleErrors(payload) {
    if ("errors" in payload && payload.errors) {
        payload.errors = payload.errors.map((e) => {
            const obj = e instanceof graphql_1.GraphQLError
                ? e.toJSON()
                : { message: e.message, ...e };
            return Object.assign(obj, {
                message: (0, grafast_1.stripAnsi)(obj.message),
                extensions: { stack: (0, grafast_1.stripAnsi)(e.stack ?? "").split("\n") },
            });
        });
    }
}
exports.handleErrors = handleErrors;
function processHeaders(headers) {
    const headerDigest = Object.create(null);
    for (const key in headers) {
        const val = headers[key];
        if (val == null) {
            continue;
        }
        if (typeof val === "string") {
            headerDigest[key] = val;
        }
        else {
            headerDigest[key] = val.join("\n");
        }
    }
    return headerDigest;
}
exports.processHeaders = processHeaders;
function getBodyFromRequest(req /* IncomingMessage */, maxLength) {
    return new Promise((resolve, reject) => {
        const chunks = [];
        let len = 0;
        const handleData = (chunk) => {
            chunks.push(chunk);
            len += chunk.length;
            if (len > maxLength) {
                req.off("end", done);
                req.off("error", reject);
                req.off("data", handleData);
                // FIXME: validate this approach
                reject(new Error("Too much data"));
            }
        };
        const done = () => {
            resolve({ type: "buffer", buffer: Buffer.concat(chunks) });
        };
        req.on("end", done);
        req.on("error", reject);
        req.on("data", handleData);
    });
}
exports.getBodyFromRequest = getBodyFromRequest;
function getBodyFromFrameworkBody(body) {
    if (typeof body === "string") {
        return {
            type: "text",
            text: body,
        };
    }
    else if (Buffer.isBuffer(body)) {
        return {
            type: "buffer",
            buffer: body,
        };
    }
    else if (typeof body === "object" && body != null) {
        return {
            type: "json",
            json: body,
        };
    }
    else {
        throw new Error(`Grafserv Express adaptor doesn't know how to interpret this request body`);
    }
}
exports.getBodyFromFrameworkBody = getBodyFromFrameworkBody;
function memo(fn) {
    let cache;
    let called = false;
    return function memoized() {
        if (called) {
            return cache;
        }
        else {
            called = true;
            cache = fn.call(this);
            return cache;
        }
    };
}
exports.memo = memo;
function normalizeRequest(request) {
    if (!request[interfaces_js_1.$$normalizedHeaders]) {
        const r = request;
        const normalized = Object.create(null);
        for (const key in r.headers) {
            normalized[key.toLowerCase()] = r.headers[key];
        }
        r[interfaces_js_1.$$normalizedHeaders] = normalized;
        r.preferJSON = Boolean(r.preferJSON);
        r.getHeader = (key) => normalized[key.toLowerCase()];
        r.getBody = memo(r.getBody);
        r.getQueryParams = memo(r.getQueryParams);
    }
    return request;
}
exports.normalizeRequest = normalizeRequest;
function httpError(statusCode, message) {
    return Object.assign(new Error(message), { statusCode });
}
exports.httpError = httpError;
//# sourceMappingURL=utils.js.map